GlowScript 2.7 VPython

#Simulador da dinâmica de um dipolo magnético interagindo com campo magnético estático
#Essa simulação precisa ser rodada em glowscript.org
#Para rodar diretamente em glowscript, acesse: https://www.glowscript.org/#/user/jocoteles/folder/RMN/program/simplesDipolo


scene = canvas(background = color.white, forward = vec(1,0,0), up = vec(0,0,1))
scene.title = '<b>Dinâmica da magnetização de um núcleo isolado</b>\n'

#{{Ctes físicas:
B0 = 10.0      #campo magnetico inical [T]
B0min = 1.0    #[T]
B0max = 20.0   #[T]
M = 10.0       #magnetizacao inicial [u.a.]
lGammaLabels = ['1H','19F','31P','13C'] #ctes giromagnéticas [labels]
lGammaValues = [42.6, 40.1, 17.2, 10.7] #ctes giromagnéticas [MHz/T]
#}}

#{{Ctes da dinâmica:
N = 500                     #número de passos por volta
T = 0.5                     #período de rotação para 1H em B0 = 10 T [s]
gamma = lGammaValues[0]     #cte giromagnética [MHz/T]
rf = 2*pi/(gamma*B0*N)      #fator de reducao: 1 volta no período T para campo B0
da = gamma*B0*rf            #tamanho do passo angular [rad]
#}}

#{{Flags de inicialização:
dragB0 = False
pausar = False
#}}

vO = vector(0,0,0)
vB0i = vector(0,0,B0)
vMi = vector(M,0,0)

aB0 = arrow(pos = vO, axis = vB0i, color = color.blue)
aM = arrow(pos = vO, axis = vMi, color = color.red)
def trailVec():
    return aM.axis
atrail = attach_trail(trailVec, radius=0.2, retain=N, type='points', color=aM.color)
atrail.stop()

#{{Movimentos do mouse
def downB0():
    global dragB0
    p = scene.mouse.pick
    if p != None:
        dragB0 = True
def moveB0():
    global dragB0
    if dragB0:
        aB0.axis = scene.mouse.pos
def upB0():
    global dragB0, B0, da
    dragB0 = False
    B0 = mag(aB0.axis)
    if B0 > B0max:
        B0 = B0max
        aB0.axis = B0*hat(aB0.axis)
    if B0 < B0min:
        B0 = B0min
        aB0.axis = B0*hat(aB0.axis)
    da = gamma*B0*rf
    sB0.value = B0
    tB0.text = '{:1.1f}'.format(B0)

scene.bind('mousedown', downB0)
scene.bind('mousemove', moveB0)
scene.bind('mouseup', upB0)
#}}

##{{Funções de binding:
def bPausar(b):
    global pausar
    pausar = not pausar
    if (b.text == 'Pausar'):
        b.text = 'Rodar'
    else:
        b.text = 'Pausar'

def bGammaCalc(g):
    global da, gamma
    for gl, gv in zip(lGammaLabels,lGammaValues):
        if gl == g.selected:
            gamma = gv
            da = gamma*B0*rf
            gtext.text = str(gamma) + ' MHz/T'

def bB0Calc(b0):
    global da, B0
    B0 = b0.value
    da = gamma*B0*rf
    aB0.axis = B0*hat(aB0.axis)
    tB0.text = '{:1.1f}'.format(B0)

def bMTrail(c):
    if c.checked:
        atrail.start()
    else:
        atrail.stop()
        atrail.clear()
#}}

button(bind=bPausar, text='Pausar', pos=scene.title_anchor)

scene.caption = 'Constante giromagnética (\\(\\gamma\\)) = '
gtext = wtext(text=str(lGammaValues[0])+' MHz/T  ', pos=scene.caption_anchor)
menu(bind=bGammaCalc, choices=lGammaLabels, selected=lGammaLabels[0], left=25)

scene.append_to_caption('\n\n')
sB0 = slider(bind=bB0Calc, min=1.0, max=20.0, value = B0, length=150, right=15)
scene.append_to_caption('Campo magnético (\\(B_{0}\\)): ')
tB0 = wtext(text='{:1.1f}'.format(sB0.value), pos=scene.caption_anchor)
scene.append_to_caption(' T')

scene.append_to_caption('\n\n')
checkbox(bind=bMTrail, text = 'Mostrar traço da magnetização', checked=False)

MathJax.Hub.Queue(["Typeset",MathJax.Hub])
while True:
    rate(N/T)
    if ((not dragB0) and (not pausar)):
        aM.rotate(angle = da, axis = aB0.axis, origin = vO)
